﻿using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace PACNET
{
    /// <summary>Backplane API</summary>
    public partial class Sys
    {
        //2.1. PAC_API void pac_EnableRetrigger(BYTE iValues)
        /// <summary>
        /// This function reboots the device.
        /// </summary>
        /// <param name="iValues">[in] Decide turning on or not.</param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_EnableRetrigger")]
        public static extern void EnableRetrigger(byte iValues);

        //2.2. PAC_API void pac_GetBackplaneID(LPSTR backplaneID)
        /// <summary>
        /// This function retrieves the backplane ID.
        /// </summary>
        /// <param name="backplaneID">[out] the pointer to a buffer to receive the backplane version.</param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetBackplaneID")]
        public static extern void GetBackplaneID(byte[] backplaneID);

        /// <summary>
        /// This function retrieves the backplane ID.
        /// </summary>
        /// <returns>Return the backplane ID.</returns> 
        public static string GetBackplaneID()
        {
            byte[] result = new byte[32];
            GetBackplaneID(result);
            string str = StrConvert.WideString(result);
            return str;
        }

        //2.3. PAC_API int pac_GetBatteryLevel(int nBattery)
        /// <summary>
        /// This function retrieves the backplane battery status.
        /// </summary>
        /// <param name="nBattery">[in] Specifies the index of battery. 
        /// 1: means first battery, 2: means second battery</param>
        /// <returns>1: means high voltage, 2: means low voltage</returns> 
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetBatteryLevel")]
        public static extern int GetBatteryLevel(int nBattery);

        //2.4. PAC_API int pac_GetDIPSwitch()
        /// <summary>
        /// This function retrieves the dip switch.
        /// </summary>
        /// <returns>Return Values of the DIP switch.</returns> 
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetDIPSwitch")]
        public static extern int GetDIPSwitch();

        //2.5. PAC_API int pac_GetSlotCount()
        /// <summary>
        /// This function retrieves the number of slot.
        /// </summary>
        /// <returns>Return the number of slots available.</returns> 
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetSlotCount")]
        public static extern int GetSlotCount();

        //2.6. PAC_API void pac_RegistryHotPlug(DWORD hWnd, DWORD msgID) (Beta testing)
        /// <summary>
        /// This function retrieves the number of slot.
        /// </summary>
        /// <param name="hWnd">[in] User specifies the handle ID.</param>
        /// <param name="msgID"></param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegistryHotPlug")]
        public static extern void RegistryHotPlug(uint hWnd, uint msgID);

        //2.7. PAC_API void pac_UnregistryHotPlug(DWORD hWnd) (Beta testing)
        /// <summary>
        /// This function deletes the registry key after turning off the hot plug.
        /// </summary>
        /// <param name="hWnd">[in] User specifies the handle ID.</param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_UnregistryHotPlug")]
        public static extern void UnregistryHotPlug(uint hWnd);
    }
}